package repository

import (
	mgo "code.30cm.net/digimon/library-go/mongo"
	"context"
	"fmt"
	"github.com/alicebob/miniredis/v2"
	"github.com/stretchr/testify/assert"
	"github.com/zeromicro/go-zero/core/stores/cache"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"testing"
	"time"
	"url_generate/internal/module/url/domain/entity"
	"url_generate/internal/module/url/domain/repository"
)

func SetupTestURLRepository(db string) (repository.URLRepository, func(), error) {
	h, p, tearDown, err := startMongoContainer()
	if err != nil {
		return nil, nil, err
	}
	s, _ := miniredis.Run()

	conf := &mgo.Conf{
		Schema:                           "mongodb",
		Host:                             fmt.Sprintf("%s:%s", h, p),
		Database:                         db,
		MaxStaleness:                     300,
		MaxPoolSize:                      100,
		MinPoolSize:                      100,
		MaxConnIdleTime:                  300,
		Compressors:                      []string{},
		EnableStandardReadWriteSplitMode: false,
		ConnectTimeoutMs:                 3000,
	}

	cacheConf := cache.CacheConf{
		cache.NodeConf{
			RedisConf: redis.RedisConf{
				Host: s.Addr(),
				Type: redis.NodeType,
			},
			Weight: 100,
		},
	}

	cacheOpts := []cache.Option{
		cache.WithExpiry(1000 * time.Microsecond),
		cache.WithNotFoundExpiry(1000 * time.Microsecond),
	}

	param := URLRepositoryParam{
		Conf:      conf,
		CacheConf: cacheConf,
		CacheOpts: cacheOpts,
	}

	repo := NewURLRepository(param)
	_, _ = repo.Index20241226001UP(context.Background())

	return repo, tearDown, nil
}

func TestAccountModel_InsertMany(t *testing.T) {
	repo, tearDown, err := SetupTestURLRepository("testDB")
	defer tearDown()
	assert.NoError(t, err)

	tests := []struct {
		name        string
		account     *entity.URLTable
		expectError bool
	}{
		{
			name: "Valid account insert",
			account: &entity.URLTable{
				ShortCode: "AAAAA",
				URL:       "https://www.google.com",
			},
			expectError: false,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 插入測試帳戶
			err := repo.InsertMany(context.Background(), []*entity.URLTable{tt.account})
			if tt.expectError {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err, "插入操作應該成功")
				one, err := repo.FindOne(context.Background(), tt.account.ShortCode)
				assert.NoError(t, err)
				assert.Equal(t, one.ShortCode, tt.account.ShortCode)
				assert.Equal(t, one.URL, tt.account.URL)
			}
		})
	}
}
