package usecase

import (
	"fmt"
	"math/big"
	"sync"
)

// IDAllocator 管理 ID 分配與釋放
type IDAllocator struct {
	bitmap *big.Int // 位圖，用於記錄 ID 使用狀態
	mu     sync.Mutex
}

// NewIDAllocator 初始化一個新的 IDAllocator
func NewIDAllocator() *IDAllocator {
	return &IDAllocator{
		bitmap: big.NewInt(0), // 初始化位圖為 0
	}
}

// Allocate 分配一個未使用的 ID
func (alloc *IDAllocator) Allocate() (string, error) {
	alloc.mu.Lock()
	defer alloc.mu.Unlock()

	// 找到第一個未使用的位
	for i := 0; i < 64; i++ {
		if alloc.bitmap.Bit(i) == 0 { // 如果該位為 0，表示可用
			alloc.bitmap.SetBit(alloc.bitmap, i, 1) // 設置該位為 1，表示已使用
			return encodeIndex(i), nil
		}
	}

	return "", fmt.Errorf("no available IDs")
}

// Release 釋放一個已使用的 ID
func (alloc *IDAllocator) Release(id string) error {
	alloc.mu.Lock()
	defer alloc.mu.Unlock()

	// 解碼 ID 得到索引
	index, err := decodeIndex(id)
	if err != nil {
		return fmt.Errorf("invalid ID: %v", err)
	}

	// 將對應位設置為 0
	alloc.bitmap.SetBit(alloc.bitmap, index, 0)

	return nil
}

// encodeIndex 將索引轉換為固定長度字符串
func encodeIndex(index int) string {
	alphabet := "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
	result := make([]byte, 5) // 固定長度為 5
	for i := 4; i >= 0; i-- {
		result[i] = alphabet[index%64]
		index /= 64
	}
	return string(result)
}

// decodeIndex 將固定長度字符串轉換回索引
func decodeIndex(id string) (int, error) {
	alphabet := "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"
	lookup := make(map[byte]int)
	for i, ch := range alphabet {
		lookup[byte(ch)] = i
	}

	index := 0
	for _, ch := range id {
		val, ok := lookup[byte(ch)]
		if !ok {
			return 0, fmt.Errorf("invalid character: %c", ch)
		}
		index = index*64 + val
	}
	return index, nil
}
