package usecase

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

func TestIDAllocator(t *testing.T) {
	allocator := NewIDAllocator()

	// 測試分配 ID
	t.Run("Allocate IDs", func(t *testing.T) {
		id1, err := allocator.Allocate()
		assert.NoError(t, err)
		assert.NotEmpty(t, id1, "Allocated ID should not be empty")

		id2, err := allocator.Allocate()
		assert.NoError(t, err)
		assert.NotEmpty(t, id2, "Allocated ID should not be empty")
		assert.NotEqual(t, id1, id2, "Allocated IDs should be unique")
	})

	// 測試釋放 ID
	t.Run("Release ID", func(t *testing.T) {
		id, err := allocator.Allocate()
		assert.NoError(t, err)

		err = allocator.Release(id)
		assert.NoError(t, err, "Releasing an allocated ID should succeed")

		// 重新分配應該得到相同的 ID
		newID, err := allocator.Allocate()
		assert.NoError(t, err)
		assert.Equal(t, id, newID, "Reallocated ID should be the same as the released ID")
	})

	// 測試釋放無效的 ID
	t.Run("Release invalid ID", func(t *testing.T) {
		err := allocator.Release("錯誤")
		assert.Error(t, err, "Releasing an invalid ID should return an error")
	})

	// 測試超過位圖容量的情況
	t.Run("Exceed bitmap capacity", func(t *testing.T) {
		// 分配 64 個 ID
		for i := 0; i < 61; i++ {
			_, err := allocator.Allocate()
			assert.NoError(t, err)
		}

		// 第 65 次分配應該返回錯誤
		_, err := allocator.Allocate()
		assert.Error(t, err, "Allocating more than 64 IDs should fail")
	})
}

func TestEncodeDecodeIndex(t *testing.T) {
	t.Run("Encode and Decode", func(t *testing.T) {
		for i := 0; i < 64; i++ {
			encoded := encodeIndex(i)
			decoded, err := decodeIndex(encoded)

			assert.NoError(t, err)
			assert.Equal(t, i, decoded, "Decoded index should match the original")
		}
	})

	t.Run("Decode invalid string", func(t *testing.T) {
		_, err := decodeIndex("不可以")
		assert.Error(t, err, "Decoding an invalid ID should return an error")
	})
}
